"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var DriverMock_exports = {};
__export(DriverMock_exports, {
  createAndStartDriverWithMockPort: () => createAndStartDriverWithMockPort,
  createAndStartTestingDriver: () => createAndStartTestingDriver
});
module.exports = __toCommonJS(DriverMock_exports);
var import_mock = require("@zwave-js/serial/mock");
var import_deferred_promise = require("alcalzone-shared/deferred-promise");
var import_promises = __toESM(require("node:fs/promises"), 1);
var import_node_os = require("node:os");
var import_node_path = __toESM(require("node:path"), 1);
var import_Driver = require("./Driver.js");
function createAndStartDriverWithMockPort(options = {}) {
  const { portAddress = "/tty/FAKE", ...driverOptions } = options;
  return new Promise(async (resolve, reject) => {
    let driver;
    let mockPort;
    import_mock.MockBinding.reset();
    import_mock.MockBinding.createPort(portAddress, {
      record: true,
      readyData: new Uint8Array()
    });
    const onSerialPortOpen = /* @__PURE__ */ __name((_port) => {
      mockPort = import_mock.MockBinding.getInstance(portAddress);
      if (!mockPort)
        reject(new Error("Mock serial port is not open!"));
      const continuePromise = (0, import_deferred_promise.createDeferredPromise)();
      resolve({
        driver,
        mockPort,
        continueStartup: /* @__PURE__ */ __name(() => continuePromise.resolve(), "continueStartup")
      });
      return continuePromise;
    }, "onSerialPortOpen");
    if (!driverOptions.logConfig) {
      driverOptions.logConfig = {
        enabled: false
      };
    }
    const testingHooks = {
      ...driverOptions.testingHooks,
      // instruct the driver to use SerialPortMock as the serialport implementation
      serialPortBinding: import_mock.SerialPortMock,
      onSerialPortOpen
    };
    driver = new import_Driver.Driver(portAddress, {
      ...driverOptions,
      testingHooks
    });
    await driver.start();
  });
}
__name(createAndStartDriverWithMockPort, "createAndStartDriverWithMockPort");
async function createAndStartTestingDriver(options = {}) {
  const { beforeStartup, skipControllerIdentification = false, skipBootloaderCheck = true, skipNodeInterview = false, loadConfiguration = true, ...internalOptions } = options;
  const testId = Math.round(Math.random() * 4294967295).toString(16).padStart(8, "0");
  internalOptions.portAddress ??= `/tty/FAKE${testId}`;
  if (skipControllerIdentification) {
    internalOptions.testingHooks ??= {};
    internalOptions.testingHooks.skipControllerIdentification = true;
  }
  if (skipNodeInterview) {
    internalOptions.testingHooks ??= {};
    internalOptions.testingHooks.skipNodeInterview = true;
  }
  if (!loadConfiguration) {
    internalOptions.testingHooks ??= {};
    internalOptions.testingHooks.loadConfiguration = false;
  }
  if (skipBootloaderCheck) {
    internalOptions.testingHooks ??= {};
    internalOptions.testingHooks.skipBootloaderCheck = true;
  }
  const cacheDir = import_node_path.default.join((0, import_node_os.tmpdir)(), "zwave-js-test-cache", testId);
  internalOptions.storage ??= {};
  internalOptions.storage.cacheDir = cacheDir;
  const { driver, continueStartup, mockPort } = await createAndStartDriverWithMockPort(internalOptions);
  if (typeof beforeStartup === "function") {
    await beforeStartup(mockPort);
  }
  const originalDestroy = driver.destroy.bind(driver);
  driver.destroy = async () => {
    await originalDestroy();
    await import_promises.default.rm(cacheDir, { recursive: true, force: true });
  };
  return new Promise((resolve) => {
    driver.once("driver ready", () => {
      resolve({ driver, mockPort });
    });
    continueStartup();
  });
}
__name(createAndStartTestingDriver, "createAndStartTestingDriver");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createAndStartDriverWithMockPort,
  createAndStartTestingDriver
});
//# sourceMappingURL=DriverMock.js.map
