"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var SerialAPICommandMachine_exports = {};
__export(SerialAPICommandMachine_exports, {
  createSerialAPICommandMachine: () => createSerialAPICommandMachine,
  getSerialAPICommandMachineConfig: () => getSerialAPICommandMachineConfig,
  getSerialAPICommandMachineOptions: () => getSerialAPICommandMachineOptions
});
module.exports = __toCommonJS(SerialAPICommandMachine_exports);
var import_serial = require("@zwave-js/serial");
var import_serialapi = require("@zwave-js/serial/serialapi");
var import_xstate = require("xstate");
function computeRetryDelay(ctx) {
  return 100 + 1e3 * (ctx.attempts - 1);
}
__name(computeRetryDelay, "computeRetryDelay");
const forwardMessage = (0, import_xstate.raise)((_, evt) => {
  const msg = evt.message;
  return {
    type: msg.type === import_serial.MessageType.Response ? "response" : "callback",
    message: msg
  };
});
function getSerialAPICommandMachineConfig(message, messageData, { timestamp, logOutgoingMessage, notifyUnsolicited, sendDataAbort }, attemptsConfig) {
  return {
    predictableActionArguments: true,
    id: "serialAPICommand",
    initial: "sending",
    context: {
      msg: message,
      data: messageData,
      attempts: 0,
      maxAttempts: attemptsConfig.controller
    },
    on: {
      // The state machine accepts any message. If it is expected
      // it will be forwarded to the correct states. If not, it
      // will be returned with the "unsolicited" event.
      message: [
        {
          cond: "isExpectedMessage",
          actions: forwardMessage
        },
        {
          actions: /* @__PURE__ */ __name((_, evt) => {
            notifyUnsolicited(evt.message);
          }, "actions")
        }
      ]
    },
    states: {
      sending: {
        // Every send attempt should increase the attempts by one
        // and remember the timestamp of transmission
        entry: [
          (0, import_xstate.assign)({
            attempts: /* @__PURE__ */ __name((ctx) => ctx.attempts + 1, "attempts"),
            txTimestamp: /* @__PURE__ */ __name((_) => timestamp(), "txTimestamp")
          }),
          (ctx) => logOutgoingMessage(ctx.msg)
        ],
        invoke: {
          id: "sendMessage",
          src: "send",
          onDone: "waitForACK",
          onError: {
            target: "retry",
            actions: (0, import_xstate.assign)({
              lastError: /* @__PURE__ */ __name((_) => "send failure", "lastError")
            })
          }
        }
      },
      waitForACK: {
        always: [{ target: "success", cond: "expectsNoAck" }],
        on: {
          CAN: {
            target: "retry",
            actions: (0, import_xstate.assign)({
              lastError: /* @__PURE__ */ __name((_) => "CAN", "lastError")
            })
          },
          NAK: {
            target: "retry",
            actions: (0, import_xstate.assign)({
              lastError: /* @__PURE__ */ __name((_) => "NAK", "lastError")
            })
          },
          ACK: "waitForResponse"
        },
        after: {
          ACK_TIMEOUT: {
            target: "retry",
            actions: (0, import_xstate.assign)({
              lastError: /* @__PURE__ */ __name((_) => "ACK timeout", "lastError")
            })
          }
        }
      },
      waitForResponse: {
        always: [
          {
            target: "waitForCallback",
            cond: "expectsNoResponse"
          }
        ],
        on: {
          response: [
            {
              target: "retry",
              cond: "responseIsNOK",
              actions: (0, import_xstate.assign)({
                lastError: /* @__PURE__ */ __name((_) => "response NOK", "lastError"),
                result: /* @__PURE__ */ __name((_, evt) => evt.message, "result")
              })
            },
            {
              target: "waitForCallback",
              actions: (0, import_xstate.assign)({
                result: /* @__PURE__ */ __name((_, evt) => evt.message, "result")
              })
            }
          ]
        },
        after: {
          // Do not retry when a response times out
          RESPONSE_TIMEOUT: [
            {
              cond: "isSendData",
              target: "waitForCallback",
              actions: [
                () => sendDataAbort(),
                (0, import_xstate.assign)({
                  lastError: /* @__PURE__ */ __name((_) => "response timeout", "lastError")
                })
              ]
            },
            {
              target: "failure",
              actions: (0, import_xstate.assign)({
                lastError: /* @__PURE__ */ __name((_) => "response timeout", "lastError")
              })
            }
          ]
        }
      },
      waitForCallback: {
        always: [{ target: "success", cond: "expectsNoCallback" }],
        on: {
          callback: [
            {
              // Preserve "response timeout" errors
              // A NOK callback afterwards is expected, but we're not interested in it
              target: "failure",
              cond: "callbackIsNOKAfterTimedOutResponse"
            },
            {
              target: "failure",
              cond: "callbackIsNOK",
              actions: (0, import_xstate.assign)({
                // Preserve "response timeout" errors
                lastError: /* @__PURE__ */ __name((_) => "callback NOK", "lastError"),
                result: /* @__PURE__ */ __name((_, evt) => evt.message, "result")
              })
            },
            {
              target: "success",
              cond: "callbackIsFinal",
              actions: (0, import_xstate.assign)({
                result: /* @__PURE__ */ __name((_, evt) => evt.message, "result")
              })
            },
            { target: "waitForCallback" }
          ]
        },
        after: {
          // Abort Send Data when it takes too long
          SENDDATA_ABORT_TIMEOUT: {
            cond: "isSendData",
            actions: [
              () => sendDataAbort()
            ]
          },
          CALLBACK_TIMEOUT: {
            target: "failure",
            actions: (0, import_xstate.assign)({
              lastError: /* @__PURE__ */ __name((_) => "callback timeout", "lastError")
            })
          }
        }
      },
      retry: {
        always: [
          { target: "retryWait", cond: "mayRetry" },
          { target: "failure" }
        ]
      },
      retryWait: {
        invoke: {
          id: "notify",
          src: "notifyRetry"
        },
        after: {
          RETRY_DELAY: "sending"
        }
      },
      success: {
        type: "final",
        data: {
          type: "success",
          txTimestamp: /* @__PURE__ */ __name((ctx) => ctx.txTimestamp, "txTimestamp"),
          result: /* @__PURE__ */ __name((ctx) => ctx.result, "result")
        }
      },
      failure: {
        type: "final",
        data: {
          type: "failure",
          reason: /* @__PURE__ */ __name((ctx) => ctx.lastError, "reason"),
          result: /* @__PURE__ */ __name((ctx) => ctx.result, "result")
        }
      }
    }
  };
}
__name(getSerialAPICommandMachineConfig, "getSerialAPICommandMachineConfig");
function getSerialAPICommandMachineOptions({ sendData, notifyRetry }, timeoutConfig) {
  return {
    services: {
      send: /* @__PURE__ */ __name((ctx) => {
        ctx.msg.markAsSent();
        return sendData(ctx.data);
      }, "send"),
      notifyRetry: /* @__PURE__ */ __name((ctx) => {
        notifyRetry(ctx.lastError, ctx.msg, ctx.attempts, ctx.maxAttempts, computeRetryDelay(ctx));
        return Promise.resolve();
      }, "notifyRetry")
    },
    guards: {
      mayRetry: /* @__PURE__ */ __name((ctx) => ctx.attempts < ctx.maxAttempts, "mayRetry"),
      isSendData: /* @__PURE__ */ __name((ctx) => (0, import_serialapi.isSendData)(ctx.msg), "isSendData"),
      expectsNoAck: /* @__PURE__ */ __name((ctx) => !ctx.msg.expectsAck(), "expectsNoAck"),
      expectsNoResponse: /* @__PURE__ */ __name((ctx) => !ctx.msg.expectsResponse(), "expectsNoResponse"),
      expectsNoCallback: /* @__PURE__ */ __name((ctx) => !ctx.msg.expectsCallback(), "expectsNoCallback"),
      isExpectedMessage: /* @__PURE__ */ __name((ctx, evt, meta) => meta.state.matches("waitForResponse") ? ctx.msg.isExpectedResponse(evt.message) : meta.state.matches("waitForCallback") ? ctx.msg.isExpectedCallback(evt.message) : false, "isExpectedMessage"),
      responseIsNOK: /* @__PURE__ */ __name((ctx, evt) => evt.type === "response" && (0, import_serial.isSuccessIndicator)(evt.message) && !evt.message.isOK(), "responseIsNOK"),
      callbackIsNOKAfterTimedOutResponse: /* @__PURE__ */ __name((ctx, evt) => evt.type === "callback" && (0, import_serial.isSuccessIndicator)(evt.message) && !evt.message.isOK() && (0, import_serialapi.isSendData)(ctx.msg) && ctx.lastError === "response timeout", "callbackIsNOKAfterTimedOutResponse"),
      callbackIsNOK: /* @__PURE__ */ __name((ctx, evt) => evt.type === "callback" && (0, import_serial.isSuccessIndicator)(evt.message) && !evt.message.isOK(), "callbackIsNOK"),
      callbackIsFinal: /* @__PURE__ */ __name((ctx, evt) => evt.type === "callback" && (!(0, import_serial.isSuccessIndicator)(evt.message) || evt.message.isOK()) && (!(0, import_serial.isMultiStageCallback)(evt.message) || evt.message.isFinal()), "callbackIsFinal")
    },
    delays: {
      RETRY_DELAY: /* @__PURE__ */ __name((ctx) => computeRetryDelay(ctx), "RETRY_DELAY"),
      RESPONSE_TIMEOUT: /* @__PURE__ */ __name((ctx) => {
        return (
          // Ask the message for its callback timeout
          ctx.msg.getResponseTimeout() || timeoutConfig.response
        );
      }, "RESPONSE_TIMEOUT"),
      CALLBACK_TIMEOUT: /* @__PURE__ */ __name((ctx) => {
        return (
          // Ask the message for its callback timeout
          ctx.msg.getCallbackTimeout() || timeoutConfig.sendDataCallback
        );
      }, "CALLBACK_TIMEOUT"),
      SENDDATA_ABORT_TIMEOUT: timeoutConfig.sendDataAbort,
      ACK_TIMEOUT: timeoutConfig.ack
    }
  };
}
__name(getSerialAPICommandMachineOptions, "getSerialAPICommandMachineOptions");
function createSerialAPICommandMachine(message, messageData, implementations, params) {
  return (0, import_xstate.createMachine)(getSerialAPICommandMachineConfig(message, messageData, implementations, params.attempts), getSerialAPICommandMachineOptions(implementations, params.timeouts));
}
__name(createSerialAPICommandMachine, "createSerialAPICommandMachine");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createSerialAPICommandMachine,
  getSerialAPICommandMachineConfig,
  getSerialAPICommandMachineOptions
});
//# sourceMappingURL=SerialAPICommandMachine.js.map
