"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var TransportServiceMachine_exports = {};
__export(TransportServiceMachine_exports, {
  createTransportServiceRXMachine: () => createTransportServiceRXMachine
});
module.exports = __toCommonJS(TransportServiceMachine_exports);
var import_xstate = require("xstate");
const receiveSegment = (0, import_xstate.assign)((ctx, evt) => {
  for (let i = evt.offset; i < evt.offset + evt.length; i++) {
    ctx.receivedBytes[i] = true;
  }
  return ctx;
});
function createTransportServiceRXMachine(implementations, params) {
  return (0, import_xstate.Machine)({
    id: "TransportServiceRX",
    initial: "waitingForSegment",
    context: {
      receivedBytes: [
        // When the machine is started, we've already received the first segment
        ...new Array(params.firstSegmentSize).fill(true),
        // The rest of the segments are still missing
        ...new Array(params.datagramSize - params.firstSegmentSize).fill(false)
      ]
    },
    states: {
      waitingForSegment: {
        always: [
          {
            cond: "isComplete",
            target: "segmentsComplete"
          },
          {
            cond: "hasHole",
            target: "segmentTimeout"
          }
        ],
        after: {
          missingSegment: "segmentTimeout"
        },
        on: {
          segment: {
            actions: receiveSegment,
            target: "waitingForSegment",
            internal: false
          }
        }
      },
      segmentTimeout: {
        invoke: {
          id: "requestMissing",
          src: "requestMissingSegment",
          onDone: {
            target: "waitingForRequestedSegment"
          },
          onError: {
            target: "failure"
          }
        }
      },
      waitingForRequestedSegment: {
        after: {
          missingSegment: "failure"
        },
        on: {
          segment: {
            actions: receiveSegment,
            target: "waitingForSegment",
            internal: false
          }
        }
      },
      segmentsComplete: {
        invoke: {
          id: "segmentsComplete",
          src: "sendSegmentsComplete",
          onDone: {
            target: "success"
          },
          onError: {
            // If sending the command fails, the node will send us the segment again
            target: "success"
          }
        }
      },
      success: {
        type: "final",
        on: {
          segment: "segmentsComplete"
        }
      },
      failure: {
        type: "final"
      }
    }
  }, {
    services: {
      requestMissingSegment: /* @__PURE__ */ __name((ctx) => {
        return implementations.requestMissingSegment(ctx.receivedBytes.indexOf(false));
      }, "requestMissingSegment"),
      sendSegmentsComplete: /* @__PURE__ */ __name(() => {
        return implementations.sendSegmentsComplete();
      }, "sendSegmentsComplete")
    },
    guards: {
      isComplete: /* @__PURE__ */ __name((ctx) => {
        return ctx.receivedBytes.every(Boolean);
      }, "isComplete"),
      hasHole: /* @__PURE__ */ __name((ctx) => ctx.receivedBytes.lastIndexOf(true) > ctx.receivedBytes.indexOf(false), "hasHole")
    },
    delays: {
      missingSegment: params.missingSegmentTimeout
    }
  });
}
__name(createTransportServiceRXMachine, "createTransportServiceRXMachine");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createTransportServiceRXMachine
});
//# sourceMappingURL=TransportServiceMachine.js.map
