"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var UpdateConfig_exports = {};
__export(UpdateConfig_exports, {
  checkForConfigUpdates: () => checkForConfigUpdates,
  installConfigUpdate: () => installConfigUpdate
});
module.exports = __toCommonJS(UpdateConfig_exports);
var import_core = require("@zwave-js/core");
var import_shared = require("@zwave-js/shared");
var import_typeguards = require("alcalzone-shared/typeguards");
var import_execa = __toESM(require("execa"), 1);
var import_promises = __toESM(require("node:fs/promises"), 1);
var import_node_os = __toESM(require("node:os"), 1);
var path = __toESM(require("node:path"), 1);
var lockfile = __toESM(require("proper-lockfile"), 1);
var import_inc = __toESM(require("semver/functions/inc.js"), 1);
var import_valid = __toESM(require("semver/functions/valid.js"), 1);
var import_max_satisfying = __toESM(require("semver/ranges/max-satisfying.js"), 1);
async function checkForConfigUpdates(currentVersion) {
  const { got } = await import("got");
  let registry;
  try {
    registry = await got.get("https://registry.npmjs.org/@zwave-js/config").json();
  } catch {
    throw new import_core.ZWaveError(`Could not check for config updates: Failed to download package information!`, import_core.ZWaveErrorCodes.Config_Update_RegistryError);
  }
  if (!(0, import_typeguards.isObject)(registry) || !(0, import_typeguards.isObject)(registry.versions)) {
    throw new import_core.ZWaveError(`Could not check for config updates: Downloaded package information does not contain version information!`, import_core.ZWaveErrorCodes.Config_Update_RegistryError);
  }
  const allVersions = Object.keys(registry.versions).filter((v) => !!(0, import_valid.default)(v)).filter((v) => /\-\d{8}$/.test(v));
  const updateRange = `>${currentVersion} <${(0, import_inc.default)(currentVersion, "patch")}`;
  const updateVersion = (0, import_max_satisfying.default)(allVersions, updateRange, {
    includePrerelease: true
  });
  if (updateVersion)
    return updateVersion;
}
__name(checkForConfigUpdates, "checkForConfigUpdates");
async function installConfigUpdate(newVersion, external) {
  const { got } = await import("got");
  let registryInfo;
  try {
    registryInfo = await got.get(`https://registry.npmjs.org/@zwave-js/config/${newVersion}`).json();
  } catch {
    throw new import_core.ZWaveError(`Config update failed: Could not fetch package info from npm registry!`, import_core.ZWaveErrorCodes.Config_Update_InstallFailed);
  }
  const url = registryInfo?.dist?.tarball;
  if (typeof url !== "string") {
    throw new import_core.ZWaveError(`Config update failed: Could not fetch package tarball URL from npm registry!`, import_core.ZWaveErrorCodes.Config_Update_InstallFailed);
  }
  const lockfilePath = external.cacheDir;
  const lockfileOptions = {
    lockfilePath: path.join(external.cacheDir, "config-update.lock")
  };
  try {
    await lockfile.lock(lockfilePath, {
      ...lockfileOptions,
      onCompromised: /* @__PURE__ */ __name(() => {
      }, "onCompromised")
    });
  } catch {
    throw new import_core.ZWaveError(`Config update failed: Another installation is already in progress!`, import_core.ZWaveErrorCodes.Config_Update_InstallFailed);
  }
  const freeLock = /* @__PURE__ */ __name(async () => {
    try {
      if (await lockfile.check(lockfilePath, lockfileOptions)) {
        await lockfile.unlock(lockfilePath, lockfileOptions);
      }
    } catch {
    }
  }, "freeLock");
  let tmpDir;
  try {
    tmpDir = await import_promises.default.mkdtemp(path.join(import_node_os.default.tmpdir(), "zjs-config-update-"));
  } catch (e) {
    await freeLock();
    throw new import_core.ZWaveError(`Config update failed: Could not create temporary directory. Reason: ${(0, import_shared.getErrorMessage)(e)}`, import_core.ZWaveErrorCodes.Config_Update_InstallFailed);
  }
  const tarFilename = path.join(tmpDir, "zjs-config-update.tgz");
  try {
    const handle = await import_promises.default.open(tarFilename, "w");
    const fstream = handle.createWriteStream({ autoClose: true });
    const response = got.stream.get(url);
    response.pipe(fstream);
    await new Promise((resolve, reject) => {
      response.on("error", reject);
      response.on("end", resolve);
    });
  } catch (e) {
    await freeLock();
    throw new import_core.ZWaveError(`Config update failed: Could not download tarball. Reason: ${(0, import_shared.getErrorMessage)(e)}`, import_core.ZWaveErrorCodes.Config_Update_InstallFailed);
  }
  function normalizeToUnixStyle(path2) {
    path2 = path2.replaceAll(":", "");
    path2 = path2.replaceAll("\\", "/");
    if (!path2.startsWith("/"))
      path2 = `/${path2}`;
    return path2;
  }
  __name(normalizeToUnixStyle, "normalizeToUnixStyle");
  const extractedDir = path.join(tmpDir, "extracted");
  try {
    await import_promises.default.rm(extractedDir, { recursive: true, force: true });
    await import_promises.default.mkdir(extractedDir, { recursive: true });
    await (0, import_execa.default)("tar", [
      "--strip-components=1",
      "-xzf",
      normalizeToUnixStyle(tarFilename),
      "-C",
      normalizeToUnixStyle(extractedDir)
    ]);
    await import_promises.default.rm(external.configDir, { recursive: true, force: true });
    await import_promises.default.mkdir(external.configDir, { recursive: true });
    await (0, import_shared.copyFilesRecursive)(path.join(extractedDir, "config"), external.configDir, (src) => src.endsWith(".json"));
    const externalVersionFilename = path.join(external.configDir, "version");
    await import_promises.default.writeFile(externalVersionFilename, newVersion, "utf8");
  } catch {
    await freeLock();
    throw new import_core.ZWaveError(`Config update failed: Could not extract tarball`, import_core.ZWaveErrorCodes.Config_Update_InstallFailed);
  }
  void import_promises.default.rm(tmpDir, { recursive: true, force: true }).catch(() => {
  });
  await freeLock();
}
__name(installConfigUpdate, "installConfigUpdate");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  checkForConfigUpdates,
  installConfigUpdate
});
//# sourceMappingURL=UpdateConfig.js.map
