"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var Values_exports = {};
__export(Values_exports, {
  V: () => V,
  defaultCCValueOptions: () => defaultCCValueOptions
});
module.exports = __toCommonJS(Values_exports);
var import_safe = require("@zwave-js/core/safe");
var import_safe2 = require("@zwave-js/shared/safe");
const defaultCCValueOptions = {
  internal: false,
  minVersion: 1,
  secret: false,
  stateful: true,
  supportsEndpoints: true,
  autoCreate: true
};
function defineStaticCCValue(commandClass, blueprint) {
  const _blueprint = {
    ...blueprint,
    options: {
      ...defaultCCValueOptions,
      ...blueprint.options
    }
  };
  const valueId = {
    commandClass,
    property: _blueprint.property,
    propertyKey: _blueprint.propertyKey
  };
  if (valueId.propertyKey === void 0)
    delete valueId.propertyKey;
  const ret = {
    get id() {
      return { ...valueId };
    },
    endpoint: /* @__PURE__ */ __name((endpoint = 0) => {
      if (!_blueprint.options.supportsEndpoints)
        endpoint = 0;
      return { ...valueId, endpoint };
    }, "endpoint"),
    is: /* @__PURE__ */ __name((testValueId) => {
      return valueId.commandClass === testValueId.commandClass && valueId.property === testValueId.property && valueId.propertyKey === testValueId.propertyKey;
    }, "is"),
    get meta() {
      return { ...import_safe.ValueMetadata.Any, ..._blueprint.meta };
    },
    get options() {
      return { ..._blueprint.options };
    }
  };
  return ret;
}
__name(defineStaticCCValue, "defineStaticCCValue");
function defineDynamicCCValue(commandClass, blueprint) {
  const options = {
    ...defaultCCValueOptions,
    ...blueprint.options
  };
  const ret = /* @__PURE__ */ __name((...args) => {
    const _blueprint = blueprint(...args);
    const actualBlueprint = {
      ..._blueprint
    };
    const valueId = {
      commandClass,
      property: actualBlueprint.property,
      propertyKey: actualBlueprint.propertyKey
    };
    if (valueId.propertyKey === void 0)
      delete valueId.propertyKey;
    const value = {
      get id() {
        return { ...valueId };
      },
      endpoint: /* @__PURE__ */ __name((endpoint = 0) => {
        if (!options.supportsEndpoints)
          endpoint = 0;
        return { ...valueId, endpoint };
      }, "endpoint"),
      get meta() {
        return { ...import_safe.ValueMetadata.Any, ...actualBlueprint.meta };
      }
    };
    return value;
  }, "ret");
  Object.defineProperty(ret, "options", {
    configurable: false,
    enumerable: true,
    get() {
      return { ...options };
    }
  });
  Object.defineProperty(ret, "is", {
    configurable: false,
    enumerable: false,
    writable: false,
    value: /* @__PURE__ */ __name((id) => id.commandClass === commandClass && blueprint.is(id), "value")
  });
  return ret;
}
__name(defineDynamicCCValue, "defineDynamicCCValue");
const V = {
  /** Defines multiple static CC values that belong to the same CC */
  defineStaticCCValues(commandClass, values) {
    return Object.fromEntries(Object.entries(values).map(([key, blueprint]) => [
      key,
      defineStaticCCValue(commandClass, blueprint)
    ]));
  },
  /** Defines multiple static CC values that belong to the same CC */
  defineDynamicCCValues(commandClass, values) {
    return Object.fromEntries(Object.entries(values).map(([key, blueprint]) => [
      key,
      defineDynamicCCValue(commandClass, blueprint)
    ]));
  },
  /** Returns a CC value definition that is named like the value `property` */
  staticProperty(property, meta, options) {
    return {
      [property]: {
        property,
        meta,
        options
      }
    };
  },
  /** Returns a CC value definition with the given name and `property` */
  staticPropertyWithName(name, property, meta, options) {
    return {
      [name]: {
        property,
        meta,
        options
      }
    };
  },
  /** Returns a CC value definition with the given name, `property` and `propertyKey` */
  staticPropertyAndKeyWithName(name, property, propertyKey, meta, options) {
    return {
      [name]: {
        property,
        propertyKey,
        meta,
        options
      }
    };
  },
  /** Returns a CC value definition with the given name and a dynamic `property` */
  dynamicPropertyWithName(name, property, is, meta, options) {
    return {
      [name]: Object.assign((...args) => ({
        property: (0, import_safe2.evalOrStatic)(property, ...args),
        meta: (0, import_safe2.evalOrStatic)(meta, ...args)
      }), { is, options })
    };
  },
  /** Returns a CC value definition with the given name and a dynamic `property` */
  dynamicPropertyAndKeyWithName(name, property, propertyKey, is, meta, options) {
    return {
      [name]: Object.assign((...args) => {
        return {
          property: (0, import_safe2.evalOrStatic)(property, ...args),
          propertyKey: (0, import_safe2.evalOrStatic)(propertyKey, ...args),
          meta: (0, import_safe2.evalOrStatic)(meta, ...args)
        };
      }, { is, options })
    };
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  V,
  defaultCCValueOptions
});
//# sourceMappingURL=Values.js.map
