"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var SerialPortBindingMock_exports = {};
__export(SerialPortBindingMock_exports, {
  CanceledError: () => CanceledError,
  MockBinding: () => MockBinding,
  MockPortBinding: () => MockPortBinding
});
module.exports = __toCommonJS(SerialPortBindingMock_exports);
var import_shared = require("@zwave-js/shared");
let ports = {};
let serialNumber = 0;
function resolveNextTick() {
  return new Promise((resolve) => process.nextTick(() => resolve()));
}
__name(resolveNextTick, "resolveNextTick");
class CanceledError extends Error {
  static {
    __name(this, "CanceledError");
  }
  canceled;
  constructor(message) {
    super(message);
    this.canceled = true;
  }
}
const MockBinding = {
  reset() {
    ports = {};
    serialNumber = 0;
  },
  // Create a mock port
  createPort(path, options = {}) {
    serialNumber++;
    const optWithDefaults = {
      echo: false,
      record: false,
      manufacturer: "The J5 Robotics Company",
      vendorId: void 0,
      productId: void 0,
      maxReadSize: 1024,
      ...options
    };
    ports[path] = {
      data: new Uint8Array(),
      // echo: optWithDefaults.echo,
      // record: optWithDefaults.record,
      readyData: optWithDefaults.readyData,
      maxReadSize: optWithDefaults.maxReadSize,
      info: {
        path,
        manufacturer: optWithDefaults.manufacturer,
        serialNumber: `${serialNumber}`,
        pnpId: void 0,
        locationId: void 0,
        vendorId: optWithDefaults.vendorId,
        productId: optWithDefaults.productId
      }
    };
  },
  async list() {
    return Object.values(ports).map((port) => port.info);
  },
  async open(options) {
    if (!options || typeof options !== "object" || Array.isArray(options)) {
      throw new TypeError('"options" is not an object');
    }
    if (!options.path) {
      throw new TypeError('"path" is not a valid port');
    }
    if (!options.baudRate) {
      throw new TypeError('"baudRate" is not a valid baudRate');
    }
    const openOptions = {
      dataBits: 8,
      lock: true,
      stopBits: 1,
      parity: "none",
      rtscts: false,
      xon: false,
      xoff: false,
      xany: false,
      hupcl: true,
      ...options
    };
    const { path } = openOptions;
    const port = ports[path];
    await resolveNextTick();
    if (!port) {
      throw new Error(`Port does not exist - please call MockBinding.createPort('${path}') first`);
    }
    if (port.openOpt?.lock) {
      throw new Error("Port is locked cannot open");
    }
    port.openOpt = { ...openOptions };
    port.instance = new MockPortBinding(port, openOptions);
    return port.instance;
  },
  getInstance(path) {
    return ports[path]?.instance;
  }
};
class MockPortBinding extends import_shared.TypedEventTarget {
  static {
    __name(this, "MockPortBinding");
  }
  openOptions;
  port;
  pendingRead;
  lastWrite;
  recording;
  writeOperation;
  isOpen;
  serialNumber;
  constructor(port, openOptions) {
    super();
    this.port = port;
    this.openOptions = openOptions;
    this.pendingRead = null;
    this.isOpen = true;
    this.lastWrite = null;
    this.recording = new Uint8Array();
    this.writeOperation = null;
    this.serialNumber = port.info.serialNumber;
    if (port.readyData) {
      const data = port.readyData;
      process.nextTick(() => {
        if (this.isOpen) {
          this.emitData(data);
        }
      });
    }
  }
  // Emit data on a mock port
  emitData(data) {
    if (!this.isOpen || !this.port) {
      throw new Error("Port must be open to pretend to receive data");
    }
    const bufferData = (0, import_shared.isUint8Array)(data) ? data : import_shared.Bytes.from(data);
    this.port.data = import_shared.Bytes.concat([this.port.data, bufferData]);
    if (this.pendingRead) {
      process.nextTick(this.pendingRead);
      this.pendingRead = null;
    }
  }
  async close() {
    if (!this.isOpen) {
      throw new Error("Port is not open");
    }
    const port = this.port;
    if (!port) {
      throw new Error("already closed");
    }
    port.openOpt = void 0;
    port.data = new Uint8Array();
    this.serialNumber = void 0;
    this.isOpen = false;
    if (this.pendingRead) {
      this.pendingRead(new CanceledError("port is closed"));
    }
    this.emit("close");
  }
  async read(buffer, offset, length) {
    if (!Buffer.isBuffer(buffer)) {
      throw new TypeError('"buffer" is not a Buffer');
    }
    if (typeof offset !== "number" || Number.isNaN(offset)) {
      throw new TypeError(`"offset" is not an integer got "${Number.isNaN(offset) ? "NaN" : typeof offset}"`);
    }
    if (typeof length !== "number" || Number.isNaN(length)) {
      throw new TypeError(`"length" is not an integer got "${Number.isNaN(length) ? "NaN" : typeof length}"`);
    }
    if (buffer.length < offset + length) {
      throw new Error("buffer is too small");
    }
    if (!this.isOpen) {
      throw new Error("Port is not open");
    }
    await resolveNextTick();
    if (!this.isOpen || !this.port) {
      throw new CanceledError("Read canceled");
    }
    if (this.port.data.length <= 0) {
      return new Promise((resolve, reject) => {
        this.pendingRead = async (err) => {
          if (err) {
            return reject(err);
          }
          try {
            const readResult = await this.read(buffer, offset, length);
            resolve(readResult);
          } catch (e) {
            reject(e);
          }
        };
      });
    }
    const lengthToRead = this.port.maxReadSize > length ? length : this.port.maxReadSize;
    const data = this.port.data.subarray(0, lengthToRead);
    buffer.set(data, offset);
    this.port.data = this.port.data.subarray(lengthToRead);
    return { bytesRead: data.length, buffer };
  }
  async write(buffer) {
    if (!(0, import_shared.isUint8Array)(buffer)) {
      throw new TypeError('"buffer" is not a Uint8Array');
    }
    if (!this.isOpen || !this.port) {
      throw new Error("Port is not open");
    }
    if (this.writeOperation) {
      throw new Error("Overlapping writes are not supported and should be queued by the serialport object");
    }
    this.writeOperation = (async () => {
      await resolveNextTick();
      if (!this.isOpen || !this.port) {
        return;
      }
      const data = this.lastWrite = import_shared.Bytes.from(buffer);
      this.emit("write", data);
      this.writeOperation = null;
    })();
    return this.writeOperation;
  }
  async update(options) {
    if (typeof options !== "object") {
      throw TypeError('"options" is not an object');
    }
    if (typeof options.baudRate !== "number") {
      throw new TypeError('"options.baudRate" is not a number');
    }
    if (!this.isOpen || !this.port) {
      throw new Error("Port is not open");
    }
    await resolveNextTick();
    if (this.port.openOpt) {
      this.port.openOpt.baudRate = options.baudRate;
    }
  }
  async set(options) {
    if (typeof options !== "object") {
      throw new TypeError('"options" is not an object');
    }
    if (!this.isOpen) {
      throw new Error("Port is not open");
    }
    await resolveNextTick();
  }
  async get() {
    if (!this.isOpen) {
      throw new Error("Port is not open");
    }
    await resolveNextTick();
    return {
      cts: true,
      dsr: false,
      dcd: false
    };
  }
  async getBaudRate() {
    if (!this.isOpen || !this.port) {
      throw new Error("Port is not open");
    }
    await resolveNextTick();
    if (!this.port.openOpt?.baudRate) {
      throw new Error("Internal Error");
    }
    return {
      baudRate: this.port.openOpt.baudRate
    };
  }
  async flush() {
    if (!this.isOpen || !this.port) {
      throw new Error("Port is not open");
    }
    await resolveNextTick();
    this.port.data = new Uint8Array();
  }
  async drain() {
    if (!this.isOpen) {
      throw new Error("Port is not open");
    }
    await this.writeOperation;
    await resolveNextTick();
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CanceledError,
  MockBinding,
  MockPortBinding
});
//# sourceMappingURL=SerialPortBindingMock.js.map
