"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var BootloaderParsers_exports = {};
__export(BootloaderParsers_exports, {
  BootloaderChunkType: () => BootloaderChunkType,
  BootloaderParser: () => BootloaderParser,
  BootloaderScreenParser: () => BootloaderScreenParser,
  bootloaderMenuPreamble: () => bootloaderMenuPreamble
});
module.exports = __toCommonJS(BootloaderParsers_exports);
var import_node_stream = require("node:stream");
var import_MessageHeaders = require("../message/MessageHeaders.js");
var BootloaderChunkType;
(function(BootloaderChunkType2) {
  BootloaderChunkType2[BootloaderChunkType2["Error"] = 0] = "Error";
  BootloaderChunkType2[BootloaderChunkType2["Menu"] = 1] = "Menu";
  BootloaderChunkType2[BootloaderChunkType2["Message"] = 2] = "Message";
  BootloaderChunkType2[BootloaderChunkType2["FlowControl"] = 3] = "FlowControl";
})(BootloaderChunkType || (BootloaderChunkType = {}));
function isFlowControl(byte) {
  return byte === import_MessageHeaders.XModemMessageHeaders.ACK || byte === import_MessageHeaders.XModemMessageHeaders.NAK || byte === import_MessageHeaders.XModemMessageHeaders.CAN || byte === import_MessageHeaders.XModemMessageHeaders.C;
}
__name(isFlowControl, "isFlowControl");
class BootloaderScreenParser extends import_node_stream.Transform {
  static {
    __name(this, "BootloaderScreenParser");
  }
  logger;
  constructor(logger) {
    super({ readableObjectMode: true });
    this.logger = logger;
  }
  receiveBuffer = "";
  flushTimeout;
  _transform(chunk, encoding, callback) {
    if (this.flushTimeout) {
      clearTimeout(this.flushTimeout);
      this.flushTimeout = void 0;
    }
    this.receiveBuffer += chunk.toString("utf8");
    this.receiveBuffer = this.receiveBuffer.replaceAll(/(error 0x)\0([0-9a-f]+)/gi, "$1$2\0");
    let nulCharIndex;
    while ((nulCharIndex = this.receiveBuffer.indexOf("\0")) > -1) {
      const screen = this.receiveBuffer.slice(0, nulCharIndex).trim();
      this.receiveBuffer = this.receiveBuffer.slice(nulCharIndex + 1);
      if (screen === "")
        continue;
      this.logger?.bootloaderScreen(screen);
      this.push(screen);
    }
    while (this.receiveBuffer.length > 0) {
      const charCode = this.receiveBuffer.charCodeAt(0);
      if (!isFlowControl(charCode))
        break;
      this.logger?.data("inbound", Uint8Array.from([charCode]));
      this.push(charCode);
      this.receiveBuffer = this.receiveBuffer.slice(1);
    }
    if (this.receiveBuffer) {
      this.flushTimeout = setTimeout(() => {
        this.flushTimeout = void 0;
        this.push(this.receiveBuffer);
        this.receiveBuffer = "";
      }, 500);
    }
    callback();
  }
}
const bootloaderMenuPreamble = "Gecko Boo";
const preambleRegex = /^Gecko Bootloader v(?<version>\d+\.\d+\.\d+)/;
const menuSuffix = "BL >";
const optionsRegex = /^(?<num>\d+)\. (?<option>.+)/gm;
class BootloaderParser extends import_node_stream.Transform {
  static {
    __name(this, "BootloaderParser");
  }
  constructor() {
    super({ objectMode: true });
  }
  _transform(chunk, encoding, callback) {
    if (typeof chunk === "number") {
      return callback(null, {
        type: BootloaderChunkType.FlowControl,
        command: chunk
      });
    }
    let screen = chunk.trim();
    const menuPreambleIndex = screen.indexOf(bootloaderMenuPreamble);
    if (menuPreambleIndex > -1 && screen.endsWith(menuSuffix)) {
      screen = screen.slice(menuPreambleIndex);
      const version = preambleRegex.exec(screen)?.groups?.version;
      if (!version) {
        return callback(null, {
          type: BootloaderChunkType.Error,
          error: "Could not determine bootloader version",
          _raw: screen
        });
      }
      const options = [];
      let match;
      while ((match = optionsRegex.exec(screen)) !== null) {
        options.push({
          num: parseInt(match.groups.num),
          option: match.groups.option
        });
      }
      this.push({
        type: BootloaderChunkType.Menu,
        _raw: screen,
        version,
        options
      });
    } else {
      this.push({
        type: BootloaderChunkType.Message,
        _raw: screen,
        message: screen
      });
    }
    callback();
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  BootloaderChunkType,
  BootloaderParser,
  BootloaderScreenParser,
  bootloaderMenuPreamble
});
//# sourceMappingURL=BootloaderParsers.js.map
